==> xpi-gmail-manager-0.5.2.tbz: lib/xpi/{582195F5-92E7-40a0-A127-DB71295901D7}/components/gmCookies.js <==
// Gmail Manager
// By Todd Long <longfocus@gmail.com>
// http://www.longfocus.com/firefox/gmanager/

const GM_CC = Components.classes;
const GM_CI = Components.interfaces;

function gmCookies()
{
  // Services
  this._console = GM_CC["@mozilla.org/consoleservice;1"].getService(GM_CI.nsIConsoleService);
  this._cookieMan = GM_CC["@mozilla.org/cookiemanager;1"].getService(GM_CI.nsICookieManager);
  this._cookieServ = GM_CC["@mozilla.org/cookieService;1"].getService(GM_CI.nsICookieService);
  this._observer = GM_CC["@mozilla.org/observer-service;1"].getService(GM_CI.nsIObserverService);
  
  // Sessions
  this._sessions = new Array();
}

gmCookies.prototype = {
  _pVer: GM_CC["@mozilla.org/xre/app-info;1"].getService(GM_CI.nsIXULAppInfo).platformVersion,
  
  loadSession: function(aHost)
  {
    // Check for host session
    if (this._sessions[aHost] != null)
    {
      // Increment connections
      this._sessions[aHost].count++;
    }
    else if (this._pVer.indexOf("1.8.1") != 0)
    {
      // Creates host session
      this._sessions[aHost] = new Object();
      this._sessions[aHost].count = 1;
      this._sessions[aHost].cookies = this._getCookies(aHost);
      
      // Load observers
      this._observer.addObserver(this, "cookie-changed", false);
      this._observer.addObserver(this, "http-on-modify-request", false);
      this._observer.addObserver(this, "http-on-examine-response", false);
    }
  },
  
  restoreSession: function(aHost)
  {
    // Check for host session
    if (this._sessions[aHost] != null)
    {
      // Decrement connections
      this._sessions[aHost].count--;
      
      if (this._sessions[aHost].count == 0)
      {
        // Remove observers
        this._observer.removeObserver(this, "cookie-changed");
        this._observer.removeObserver(this, "http-on-modify-request");
        this._observer.removeObserver(this, "http-on-examine-response");
        
        // Restore cookies
        this._putCookies(aHost, this._sessions[aHost].cookies);
        
        // Remove host session
        delete this._sessions[aHost];
      }
    }
  },
  
  _getCookies: function(aHost)
  {
    var iioServ = GM_CC["@mozilla.org/network/io-service;1"].getService(GM_CI.nsIIOService);
    var cookieEnum = this._cookieMan.enumerator;
    var cookies = new Array();
    
    while (cookieEnum.hasMoreElements())
    {
      var nextCookie = cookieEnum.getNext().QueryInterface(GM_CI.nsICookie2);
      
      if (nextCookie.host.indexOf(aHost) > -1)
      {
        var pair = nextCookie.name + "=" + nextCookie.value + ";";
        var type = (nextCookie.isDomain ? "Domain=" : "Host=") + nextCookie.host + ";";
        var path = "Path=" + nextCookie.path + ";";
        var expires = (nextCookie.isSession ? "" : "Expires=" + (new Date(nextCookie.expires * 1000).toGMTString()));
        
        // Creates cookie
        cookies[nextCookie.name] = new Object();
        cookies[nextCookie.name].set = pair + type + path + expires;
        cookies[nextCookie.name].uri = iioServ.newURI("http://" + nextCookie.rawHost, "UTF-8", null);
        cookies[nextCookie.name].value = nextCookie.value;
      }
    }
    
    return cookies;
  },
  
  _putCookies: function(aHost, aCookies)
  {
    var cookieEnum = this._cookieMan.enumerator;
    
    // Clear host cookies
    while (cookieEnum.hasMoreElements())
    {
      var cookie = cookieEnum.getNext().QueryInterface(GM_CI.nsICookie2);
      var remove = (cookie.name in aCookies ? cookie.value != aCookies[cookie.name].value : true);
      
      if (cookie.host.indexOf(aHost) > -1 && remove)
        this._cookieMan.remove(cookie.host, cookie.name, cookie.path, false);
    }
    
    // Restore host cookies
    for (var name in aCookies)
      this._cookieServ.setCookieString(aCookies[name].uri, null, aCookies[name].set, null);
  },
  
  _inSessions: function(aHost)
  {
    var host = null;
    
    for (var lookupHost in this._sessions)
      if (aHost.indexOf(lookupHost) > -1)
        host = lookupHost;
    
    return host;
  },
  
  observe: function(aSubject, aTopic, aData)
  {
    switch (aTopic)
    {
      case "cookie-changed":
      {
        var inCookie = aSubject.QueryInterface(GM_CI.nsICookie2);
        var host = this._inSessions(inCookie.host);
        
        if (host != null)
        {
          var myCookie = this._sessions[host].cookies[inCookie.name];
          
          if (myCookie && myCookie.value != inCookie.value) // && myCookie.set.split("="))[0] != "S")
          {
            this._cookieServ.setCookieString(myCookie.uri, null, myCookie.set, null);
            
            //this._console.logStringMessage(myCookie.uri.host + " : " + myCookie.set);
            //this._console.logStringMessage(aData);
            //this._console.logStringMessage("---");
          }
        }
        
        break;
      }
      case "http-on-modify-request":
      case "http-on-examine-response":
      {
        var httpChannel = aSubject.QueryInterface(GM_CI.nsIHttpChannel);
        var uri = httpChannel.URI;
        var host = this._inSessions(uri.host);
        var isGood = (uri.originCharset.indexOf("gmanager") == -1 && host != null);
        
        if (aTopic == "http-on-modify-request" && isGood)
        {
          try {
            var session = this._sessions[host];
            
            // Clear any cookies
            httpChannel.setRequestHeader("Cookie", "", false);
            
            for (var name in session.cookies)
            {
              var cookieValue = (session.cookies[name].set.split(";"))[0];
              var cookieHost = session.cookies[name].uri.host;
              
              //if (name == "GMAIL_AT")
                //httpChannel.URI.path = uri.path.replace(/\&at=[^&$]*/, "&at=" + (cookieValue.split("="))[1]);
              
              if (uri.host.indexOf(cookieHost) > -1)
                httpChannel.setRequestHeader("Cookie", cookieValue, true);
            }
          } catch(e) {}
        }
        else if (aTopic == "http-on-examine-response" && isGood)
        {
          try {
            var cookies = httpChannel.getResponseHeader("Set-Cookie").split("\n");
            
            for (var i = 0; i < cookies.length; i++)
            {
              var pair = (cookies[i].split(";"))[0];
              var name = (pair.split("="))[0];
              
              this._sessions[host].cookies[name] = new Object();
              this._sessions[host].cookies[name].set = cookies[i];
              this._sessions[host].cookies[name].uri = uri;
              this._sessions[host].cookies[name].value = (pair.split("="))[1];
            }
          } catch(e) {}
        }
        
        break;
      }
    }
  },
  
  QueryInterface: function(iid)
  {
    if (!iid.equals(GM_CI.gmICookies) && 
        !iid.equals(GM_CI.nsISupports))
      throw Components.results.NS_ERROR_NO_INTERFACE;
    return this;
  }
}

var myModule = {
  firstTime: true,
  
  myCID: Components.ID("{81516840-f7dd-11da-974d-0800200c9a66}"),
  myDesc: "Cookies Service",
  myProgID: "@longfocus.com/gmanager/cookies;1",
  myFactory: {
    createInstance: function (outer, iid) {
      if (outer != null)
        throw Components.results.NS_ERROR_NO_AGGREGATION;
      
      return (new gmCookies()).QueryInterface(iid);
    }
  },

  registerSelf: function (compMgr, fileSpec, location, type)
  {
    if (this.firstTime) {
      this.firstTime = false;
      throw Components.results.NS_ERROR_FACTORY_REGISTER_AGAIN;
    }
    
    compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
    compMgr.registerFactoryLocation(this.myCID, this.myDesc, this.myProgID, fileSpec, location, type);
  },
 
  getClassObject: function (compMgr, cid, iid)
  {
    if (!cid.equals(this.myCID))
      throw Components.results.NS_ERROR_NO_INTERFACE;
    
    if (!iid.equals(Components.interfaces.nsIFactory))
      throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    
    return this.myFactory;
  },
  
  canUnload: function(compMgr) { return true; }
};

function NSGetModule(compMgr, fileSpec) { return myModule; }

==> xpi-gmail-manager-0.5.3.tbz: lib/xpi/{582195F5-92E7-40a0-A127-DB71295901D7}/components/gmCookies.js <==
// Gmail Manager
// By Todd Long <longfocus@gmail.com>
// http://www.longfocus.com/firefox/gmanager/

const GM_CC = Components.classes;
const GM_CI = Components.interfaces;

function gmCookies()
{
  // Services
  this._console = GM_CC["@mozilla.org/consoleservice;1"].getService(GM_CI.nsIConsoleService);
  this._cookieMan = GM_CC["@mozilla.org/cookiemanager;1"].getService(GM_CI.nsICookieManager);
  this._cookieServ = GM_CC["@mozilla.org/cookieService;1"].getService(GM_CI.nsICookieService);
  this._observer = GM_CC["@mozilla.org/observer-service;1"].getService(GM_CI.nsIObserverService);
  
  try {
    this._pVer = GM_CC["@mozilla.org/xre/app-info;1"].getService(GM_CI.nsIXULAppInfo).platformVersion;
  } catch(e) {
    this._pVer = null;
  }
  
  // Sessions
  this._sessions = new Array();
}

gmCookies.prototype = {
  loadSession: function(aHost)
  {
    // Check for host session
    if (this._sessions[aHost] != null)
    {
      // Increment connections
      this._sessions[aHost].count++;
    }
    else if (this._pVer == null || this._pVer.indexOf("1.8.1") != 0)
    {
      // Creates host session
      this._sessions[aHost] = new Object();
      this._sessions[aHost].count = 1;
      this._sessions[aHost].cookies = this._getCookies(aHost);
      
      // Load observers
      this._observer.addObserver(this, "cookie-changed", false);
      this._observer.addObserver(this, "http-on-modify-request", false);
      this._observer.addObserver(this, "http-on-examine-response", false);
    }
  },
  
  restoreSession: function(aHost)
  {
    // Check for host session
    if (this._sessions[aHost] != null)
    {
      // Decrement connections
      this._sessions[aHost].count--;
      
      if (this._sessions[aHost].count == 0)
      {
        // Remove observers
        this._observer.removeObserver(this, "cookie-changed");
        this._observer.removeObserver(this, "http-on-modify-request");
        this._observer.removeObserver(this, "http-on-examine-response");
        
        // Restore cookies
        this._putCookies(aHost, this._sessions[aHost].cookies);
        
        // Remove host session
        delete this._sessions[aHost];
      }
    }
  },
  
  _getCookies: function(aHost)
  {
    var iioServ = GM_CC["@mozilla.org/network/io-service;1"].getService(GM_CI.nsIIOService);
    var cookieEnum = this._cookieMan.enumerator;
    var cookies = new Array();
    
    while (cookieEnum.hasMoreElements())
    {
      var nextCookie = cookieEnum.getNext().QueryInterface(GM_CI.nsICookie2);
      
      if (nextCookie.host.indexOf(aHost) > -1)
      {
        var pair = nextCookie.name + "=" + nextCookie.value + ";";
        var type = (nextCookie.isDomain ? "Domain=" : "Host=") + nextCookie.host + ";";
        var path = "Path=" + nextCookie.path + ";";
        var expires = (nextCookie.isSession ? "" : "Expires=" + (new Date(nextCookie.expires * 1000).toGMTString()));
        
        // Creates cookie
        cookies[nextCookie.name] = new Object();
        cookies[nextCookie.name].set = pair + type + path + expires;
        cookies[nextCookie.name].uri = iioServ.newURI("http://" + nextCookie.rawHost, "UTF-8", null);
        cookies[nextCookie.name].value = nextCookie.value;
      }
    }
    
    return cookies;
  },
  
  _putCookies: function(aHost, aCookies)
  {
    var cookieEnum = this._cookieMan.enumerator;
    
    // Clear host cookies
    while (cookieEnum.hasMoreElements())
    {
      var cookie = cookieEnum.getNext().QueryInterface(GM_CI.nsICookie2);
      var remove = (cookie.name in aCookies ? cookie.value != aCookies[cookie.name].value : true);
      
      if (cookie.host.indexOf(aHost) > -1 && remove)
        this._cookieMan.remove(cookie.host, cookie.name, cookie.path, false);
    }
    
    // Restore host cookies
    for (var name in aCookies)
      this._cookieServ.setCookieString(aCookies[name].uri, null, aCookies[name].set, null);
  },
  
  _inSessions: function(aHost)
  {
    var host = null;
    
    for (var lookupHost in this._sessions)
      if (aHost.indexOf(lookupHost) > -1)
        host = lookupHost;
    
    return host;
  },
  
  observe: function(aSubject, aTopic, aData)
  {
    switch (aTopic)
    {
      case "cookie-changed":
      {
        var inCookie = aSubject.QueryInterface(GM_CI.nsICookie2);
        var host = this._inSessions(inCookie.host);
        
        if (host != null)
        {
          var myCookie = this._sessions[host].cookies[inCookie.name];
          
          if (myCookie && myCookie.value != inCookie.value) // && myCookie.set.split("="))[0] != "S")
          {
            this._cookieServ.setCookieString(myCookie.uri, null, myCookie.set, null);
            
            //this._console.logStringMessage(myCookie.uri.host + " : " + myCookie.set);
            //this._console.logStringMessage(aData);
            //this._console.logStringMessage("---");
          }
        }
        
        break;
      }
      case "http-on-modify-request":
      case "http-on-examine-response":
      {
        var httpChannel = aSubject.QueryInterface(GM_CI.nsIHttpChannel);
        var uri = httpChannel.URI;
        var host = this._inSessions(uri.host);
        var isGood = (uri.originCharset.indexOf("gmanager") == -1 && host != null);
        
        if (aTopic == "http-on-modify-request" && isGood)
        {
          try {
            var session = this._sessions[host];
            
            // Clear any cookies
            httpChannel.setRequestHeader("Cookie", "", false);
            
            for (var name in session.cookies)
            {
              var cookieValue = (session.cookies[name].set.split(";"))[0];
              var cookieHost = session.cookies[name].uri.host;
              
              //if (name == "GMAIL_AT")
                //httpChannel.URI.path = uri.path.replace(/\&at=[^&$]*/, "&at=" + (cookieValue.split("="))[1]);
              
              if (uri.host.indexOf(cookieHost) > -1)
                httpChannel.setRequestHeader("Cookie", cookieValue, true);
            }
          } catch(e) {}
        }
        else if (aTopic == "http-on-examine-response" && isGood)
        {
          try {
            var cookies = httpChannel.getResponseHeader("Set-Cookie").split("\n");
            
            for (var i = 0; i < cookies.length; i++)
            {
              var pair = (cookies[i].split(";"))[0];
              var name = (pair.split("="))[0];
              
              this._sessions[host].cookies[name] = new Object();
              this._sessions[host].cookies[name].set = cookies[i];
              this._sessions[host].cookies[name].uri = uri;
              this._sessions[host].cookies[name].value = (pair.split("="))[1];
            }
          } catch(e) {}
        }
        
        break;
      }
    }
  },
  
  QueryInterface: function(iid)
  {
    if (!iid.equals(GM_CI.gmICookies) && 
        !iid.equals(GM_CI.nsISupports))
      throw Components.results.NS_ERROR_NO_INTERFACE;
    return this;
  }
}

var myModule = {
  firstTime: true,
  
  myCID: Components.ID("{81516840-f7dd-11da-974d-0800200c9a66}"),
  myDesc: "Cookies Service",
  myProgID: "@longfocus.com/gmanager/cookies;1",
  myFactory: {
    createInstance: function (outer, iid) {
      if (outer != null)
        throw Components.results.NS_ERROR_NO_AGGREGATION;
      
      return (new gmCookies()).QueryInterface(iid);
    }
  },

  registerSelf: function (compMgr, fileSpec, location, type)
  {
    if (this.firstTime) {
      this.firstTime = false;
      throw Components.results.NS_ERROR_FACTORY_REGISTER_AGAIN;
    }
    
    compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
    compMgr.registerFactoryLocation(this.myCID, this.myDesc, this.myProgID, fileSpec, location, type);
  },
 
  getClassObject: function (compMgr, cid, iid)
  {
    if (!cid.equals(this.myCID))
      throw Components.results.NS_ERROR_NO_INTERFACE;
    
    if (!iid.equals(Components.interfaces.nsIFactory))
      throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    
    return this.myFactory;
  },
  
  canUnload: function(compMgr) { return true; }
};

function NSGetModule(compMgr, fileSpec) { return myModule; }
